/* 
 * Copyright (C) 2000  Florian Sander
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

static int cmd_savestats(struct userrec *u, int idx, char *par)
{
  write_stats();
  putlog(LOG_CMDS, "*", "#%s# savestats", dcc[idx].nick);
  return 0;
}

static int cmd_loadstats(struct userrec *u, int idx, char *par)
{
  read_stats();
  putlog(LOG_CMDS, "*", "#%s# loadstats", dcc[idx].nick);
  return 0;
}

static int cmd_writewebstats(struct userrec *u, int idx, char *par)
{
  Context;
  writewebstats(par);
  putlog(LOG_CMDS, "*", "#%s# writewebstats %s", dcc[idx].nick, par);
  Context;
  return 0;
}

static int cmd_purgestats(struct userrec *u, int idx, char *par)
{
  Context;
  purgestats();
  Context;
  deloldstatusers();
  Context;
  putlog(LOG_CMDS, "*", "#%s# purgestats", dcc[idx].nick);
  Context;
  return 0;
}

static int cmd_sumuser(struct userrec *user, int idx, char *par)
{
  struct userrec *u1, *u2;
  char *user1, *user2;
  globstats *gs;
  locstats *ls;
  int i;
  struct list_type *q;

  Context;
  user1 = newsplit(&par);
  user2 = par;
  u1 = get_user_by_handle(userlist, user1);
  u2 = get_user_by_handle(userlist, user2);
  if (!user1 || !user2 || !u1 || !u2) {
    dprintf(idx, "Usage: .sumuser <user1> <user2>\n");
    return 0;
  }
  for (gs = sdata; gs; gs = gs->next) {
    for (ls = gs->local; ls; ls = ls->next) {
      if (!strcasecmp(ls->user, u2->handle)) {
        for (i = 0; i < TOTAL_TYPES; i++)
          incrstats(u1->handle, gs->chan, i, ls->values[S_TOTAL][i], 0);
        if (getstats(u1->handle, "", "gstarted", 0)
            < getstats(u2->handle, "", "gstarted", 0))
          incrstats(u1->handle, "", T_LSTARTED, getstats(u2->handle, "", "gstarted", 0), 1);
      }
    }
  }
  for (q = get_user(&USERENTRY_HOSTS, u2); q; q = q->next)
    addhost_by_handle(u1->handle, q->extra);
  deluser(u2->handle);
  dprintf(idx, "Transferred stats from %s to %s and deleted %s\n", user2,
  	  u1->handle, user2);
  putlog(LOG_CMDS, "*", "#%s# sumuser %s %s", dcc[idx].nick, user1, user2);
  Context;
  return 0;
}

static int cmd_resetuser(struct userrec *u, int idx, char *par)
{
  char *user, *chan;
  locstats *ls;

  Context;
  user = newsplit(&par);
  chan = newsplit(&par);
  ls = findlocstats(chan, user);
  if (!ls) {
    dprintf(idx, "Couldn't find stats for %s in %s.", user, chan);
    return 0;
  }
  resetlocstats(ls);
  putlog(LOG_CMDS, "*", "#%s# resetuser %s %s", dcc[idx].nick, user, chan);
  Context;
  return 0;
}

static cmd_t mydcc[] =
{
  {"savestats", "m", cmd_savestats, NULL},
  {"loadstats", "m", cmd_loadstats, NULL},
  {"writewebstats", "m|-", cmd_writewebstats, NULL},
  {"purgestats", "m|-", cmd_purgestats, NULL},
  {"sumuser", "n|-", cmd_sumuser, NULL},
  {"resetuser", "m|-", cmd_resetuser, NULL},
  {0, 0, 0, 0}
};
