/* 
 * Copyright (C) 2000  Florian Sander
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

static struct dcc_table LIVESTATS_LISTEN =
{
  "LIVESTATS_LISTEN",
  DCT_VALIDIDX,
  eof_livestats,
  livestats_accept,
  0,
  timeout_listen_livestats,
  display_livestats_accept,
  0,
  NULL,
  0
};

static struct dcc_table LIVESTATS =
{
  "LIVESTATS",
  DCT_VALIDIDX,
  eof_livestats,
  livestats_activity,
  &livestats_timeout,
  timeout_livestats,
  display_livestats,
  expmem_livestats,
  kill_livestats,
#ifdef OLDBOT
  out_livestats,
#else
  out_livestats,
  outdone_livestats
#endif
};

static void start_listen_livestats(int port)
{
  int i, zz;
  
  stop_listen_livestats();
  zz = open_listen(&port);
  if (zz == (-1)) {
    putlog(LOG_MISC, "*", "ERROR! Cannot open listening socket for livestats!");
    return;
  }
  if ((i = new_dcc(&LIVESTATS_LISTEN, 0)) == -1) {
    putlog(LOG_MISC, "*", "ERROR! Cannot open listening socket for livestats! DCC table is full!");
    return;
  }
  dcc[i].sock = zz;
  dcc[i].addr = (IP) (-559026163);
  dcc[i].port = port;
  strcpy(dcc[i].nick, "livestats");
  strcpy(dcc[i].host, "*");
  dcc[i].timeval = now;
  putlog(LOG_MISC, "*", "Now listening for livestats connections on port %d", port);
}

static void stop_listen_livestats()
{
  int i;

  for (i = 0; i < dcc_total; i++) {
    if (dcc[i].type == &LIVESTATS_LISTEN) {
      putlog(LOG_MISC, "*",
      	     "no longer listening for livestats connections on port %d",
             dcc[i].port);
      killsock(dcc[i].sock);
      lostdcc(i);
    } else if (dcc[i].type == &LIVESTATS) {
      putlog(LOG_MISC, "*", "killing livestats connection from %s", dcc[i].host);
      killsock(dcc[i].sock);
      lostdcc(i);
    }
  }
}

static void livestats_activity(int idx, char *buf, int len)
{
  char *cmd, *path;
  int lev;

  if (!strncasecmp(buf, "GET ", 4)) {
    if (!stats_info_access(idx)->cmd) {
      stats_info_access(idx)->cmd = nmalloc(strlen(buf) + 1);
      strcpy(stats_info_access(idx)->cmd, buf);
    }
  }
  cmd = newsplit(&buf);
  if (!strcmp(cmd, "GET")) {
    lev = logmodes(stats_loglevel);
    if (lev)
      putlog(lev, "*", "%s: GET %s", dcc[idx].host, buf);
    dprintf(idx, "HTTP/1.0 200 OK\nContent-Type: text/html\n\n");
    path = newsplit(&buf);
    send_livestats(idx, path);
  } else if (!strcasecmp(cmd, "User-Agent:")) {
    if (stats_info_access(idx)->browser)
      return;
    stats_info_access(idx)->browser = nmalloc(strlen(buf) + 1);
    strcpy(stats_info_access(idx)->browser, buf);
  } else if (!strcasecmp(cmd, "Referer:")) {
    if (stats_info_access(idx)->referer)
      return;
    stats_info_access(idx)->referer = nmalloc(strlen(buf) + 1);
    strcpy(stats_info_access(idx)->referer, buf);
  } else if (!buf[0]) {
    dcc[idx].status = 1;
#ifndef OLDBOT
    /* If there's no data in our socket, we immediately get rid of it.
     */
    if (!sock_has_data(SOCK_DATA_OUTGOING, dcc[idx].sock)) {
      killsock(dcc[idx].sock);
      lostdcc(idx);
    }
#endif
  }
}

#ifndef OLDBOT
static void outdone_livestats(int idx)
{
  if (dcc[idx].status) {
    killsock(dcc[idx].sock);
    lostdcc(idx);
  } else
    dcc[idx].status = 1;
}
#endif

static void display_livestats(int idx, char *buf)
{
  sprintf(buf, "livestats");
}

static void display_livestats_accept(int idx, char *buf)
{
  sprintf(buf, "lstn port");
}

static void timeout_livestats(int idx)
{
  killsock(dcc[idx].sock);
  lostdcc(idx);
}

static void timeout_listen_livestats(int idx)
{
  debug0("timeout listen");
  killsock(dcc[idx].sock);
  lostdcc(idx);
}

static void kill_livestats(int idx, void *x)
{
  register struct stats_clientinfo *p = (struct stats_clientinfo *) x;
  char ts[41], test[11];
  time_t tt;
  FILE *f;

  Context;
  tt = now;
  if (!p) {
    putlog(LOG_MISC, "*", "Can't kill clientinfo, no pointer. This should not happen!");
    return;
  }
  ctime(&tt);
  /* 05/Feb/2000:12:08:17 +0100 */
  strftime(test, 19, "%z", localtime(&tt));
  if (test[0] != 'z')
    strftime(ts, 40, "%d/%b/%Y:%H:%M:%S %z", localtime(&tt));
  else
    strftime(ts, 40, "%d/%b/%Y:%H:%M:%S", localtime(&tt));
  if (livestats_log[0]) {
    f = fopen(livestats_log, "a");
    if (f == NULL)
      putlog(LOG_MISC, "*", "ERROR writing livestats log.");
    else {
      if (test[0] != 'z')
        fprintf(f,
    	 "%s - - [%s] \"%s\" %d %d \"%s\" \"%s\"\n", dcc[idx].host, ts,
	 p->cmd ? p->cmd : "", p->code, p->traffic,
	 p->referer ? p->referer : "-", p->browser ?  p->browser : "");
      else
        fprintf(f,
    	 "%s - - [%s %+05d] \"%s\" %d %d \"%s\" \"%s\"\n", dcc[idx].host,
	 ts, offset * (-1) * 100, p->cmd   ? p->cmd : "", p->code ,p->traffic,
	 p->referer ? p->referer : "-", p->browser ?  p->browser : "");
      fclose(f);
    }
  }
  if (p->browser)
    nfree(p->browser);
  if (p->referer)
    nfree(p->referer);
  if (p->cmd)
    nfree(p->cmd);
  nfree(p);
}

static int expmem_livestats(void *x)
{
  register struct stats_clientinfo *p = (struct stats_clientinfo *) x;
  int tot = sizeof(struct stats_clientinfo);

  Context;
  if (!p) {
    putlog(LOG_MISC, "*", "Can't expmem clientinfo, no pointer. This should not happen!");
    return 0;
  }
  if (p->browser)
    tot += strlen(p->browser) + 1;
  if (p->referer)
    tot += strlen(p->referer) + 1;
  if (p->cmd)
    tot += strlen(p->cmd) + 1;
  return tot;
}

static void out_livestats(int idx, char *buf, void *x)
{
  register struct stats_clientinfo *p = (struct stats_clientinfo *) x;

  if (!p) {
    putlog(LOG_MISC, "*", "No stats_clientinfo pointer. This should not happen!");
    return;
  }
  p->traffic += strlen(buf);
  tputs(dcc[idx].sock, buf, strlen(buf));
}

static void livestats_accept(int idx, char *buf, int len)
{
  unsigned long ip;
  unsigned short port;
  int j = 0, sock, i;
  char s[UHOSTLEN];

  Context;
  if (dcc_total + 1 >= max_dcc) {
    j = answer(dcc[idx].sock, s, &ip, &port, 0);
    if (j != -1) {
      dprintf(-j, "Sorry, too many connections already.\r\n");
      killsock(j);
    }
    return;
  }
  sock = answer(dcc[idx].sock, s, &ip, &port, 0);
  if (sock < 0) {
    neterror(s);
    putlog(LOG_MISC, "*", "Stats.mod: Error accepting livestats connection: %s", s);
    return;
  }
  if ((i = new_dcc(&LIVESTATS, sizeof(struct stats_clientinfo))) == (-1)) {
    putlog(LOG_MISC, "*", "Error accepting livestats connection. DCC table is full.");
    killsock(sock);
    return;
  }
  dcc[i].sock = sock;
  dcc[i].addr = ip;
  dcc[i].port = port;
  strcpy(dcc[i].nick, "httpstats");
#ifndef OLDBOT
  sprintf(s, "%s", iptostr(my_htonl(ip)));
#else
  sprintf(s, "%lu.%lu.%lu.%lu", (ip >> 24) & 0xff, (ip >> 16) & 0xff,
  	  (ip >> 8) & 0xff, ip & 0xff); /* dw */
#endif
  strcpy(dcc[i].host, s);
  dcc[i].timeval = now;
  dcc[i].status = 0;
  ((struct stats_clientinfo *) dcc[i].u.other)->traffic = 0;
  ((struct stats_clientinfo *) dcc[i].u.other)->code = 200;
  ((struct stats_clientinfo *) dcc[i].u.other)->browser = NULL;
  ((struct stats_clientinfo *) dcc[i].u.other)->referer = NULL;
  ((struct stats_clientinfo *) dcc[i].u.other)->cmd = NULL;
}

static void eof_livestats(int idx)
{
  debug0("eof accept");
  killsock(dcc[idx].sock);
  lostdcc(idx);
}

static void send_livestats(int idx, char *buf)
{
  char *channel, *command;
  char what[512], what2[512], *pwhat, *pwhat2, *type, *type2, *stoday, *user, ts[61];
  char bground[140];
  struct chanset_t *chan;
  int today = 0;
  globstats *gs;
  locstats *ls, *ls2;
  time_t tt, ttbuf;
  struct userrec *u;
  int itype, itype2, nr, i;
  int total, rest, width, max;
  float onep, percent, wpercent;

  ttbuf = now;
  tt = now;
  dprintf(idx, "<html><title>Livestats of %s</title>\n", botnetnick);
  if (!strcmp(buf, "/")) {
    dprintf(idx, "%s<br><br><br>\n", bodytag);
    dprintf(idx, "<table border=1 width=100%%>\n");
    for (gs = sdata; gs; gs = gs->next) {
      if (gs->chan[0] == '#') {
#ifndef OLDBOT
        chan = findchan_by_dname(gs->chan);
#else
        chan = findchan(gs->chan);
#endif
        if (chan && chan->channel.topic)
          dprintf(idx, "<tr><td rowspan=3 align=center width=50%%><a href=\"/%s/\">%s</a><br><font size=-1>(%s)</font></td>",
                  gs->chan + 1, gs->chan, chan->channel.topic);
        else
          dprintf(idx, "<tr><td rowspan=3 align=center width=50%%><a href=\"/%s/\">%s</a></td>",
                  gs->chan + 1, gs->chan);
        dprintf(idx, "<td rowspan=2 align=center>top</td>", gs->chan + 1);
        dprintf(idx, "<td align=center><a href=\"%s/top/total/words/\">total</a></td>", gs->chan + 1);
        dprintf(idx, "</tr>\n<tr>");
        dprintf(idx, "<td align=center><a href=\"%s/top/today/words/\">today</a></td>", gs->chan + 1);
        dprintf(idx, "</tr>\n<tr>");
        dprintf(idx, "<td align=center colspan=2><a href=\"%s/users/\">users</a></td>", gs->chan + 1);
        /* dprintf(idx, "</tr></table>\n"); */
        dprintf(idx, "</tr><tr><td colspan=3 height=0><font size=-5>&nbsp;</font></td></tr>\n");
      }
    }
    dprintf(idx, "<tr><th colspan=3 align=center>");
    dprintf(idx, "<a href=\"http://www.visions-of-fantasy.de/stats.mod/\">");
    dprintf(idx, "Stats.mod v%s</a></font></th></tr></table>\n", MODULE_VERSION);
    dprintf(idx, "</body></html>\n");
    return;
  }
  if (buf[0] == '/')
    buf[0] = '#';
  if (buf[strlen(buf) - 1] == '/')
    buf[strlen(buf) - 1] = 0;
  channel = splitpath(&buf);
#ifndef OLDBOT
  if (!findchan_by_dname(channel)) {
#else
  if (!findchan(channel)) {
#endif
    dprintf(idx, "no such channel %s", channel);
    stats_info_access(idx)->code = 404;
    return;
  }
  if (!strcmp(buf, "/") || !buf[0]) {
    dprintf(idx, "%s<br><br><br><center>\n", bodytag);
    dprintf(idx, "<table border=1 width=50%%>\n");
    dprintf(idx, "<tr><th colspan=2><font size=+2>%s</font></th></tr>", channel);
    dprintf(idx, "<tr><td rowspan=2 align=center>top</td>");
    dprintf(idx, "<td align=center><a href=\"top/total/words/\">total</a></td>");
    dprintf(idx, "</tr>\n<tr>");
    dprintf(idx, "<td align=center><a href=\"top/today/words/\">today</a></td>");
    dprintf(idx, "</tr>\n<tr>");
    dprintf(idx, "<td align=center colspan=2><a href=\"users/\">users</a></td>");
    dprintf(idx, "</tr><tr><td colspan=2 height=0><font size=-5>&nbsp;</font></td></tr>\n");
    dprintf(idx, "<tr><th colspan=2 align=center><font size=-2>");
    dprintf(idx, "<a href=\"http://www.visions-of-fantasy.de/stats.mod/\">");
    dprintf(idx, "Stats.mod v%s</a></font></th></tr></table>\n", MODULE_VERSION);
    dprintf(idx, "</center></body></html>\n");
    return;
  }
  command = splitpath(&buf);
  if (!strcasecmp(command, "top")) {
    if (!strcmp(buf, "/") || !buf[0]) {
      dprintf(idx, "<body><table>");
      dprintf(idx, "<tr><td><a href=\"total/\">total</a></td></tr>");
      dprintf(idx, "<tr><td><a href=\"today/\">today</a></td></tr>");
      dprintf(idx, "</table></body></html>");
      return;
    }
    stoday = splitpath(&buf);
    if (!strcasecmp(stoday, "today"))
      today = 1;
    else if (!strcasecmp(stoday, "total"))
      today = 0;
    else {
      dprintf(idx, "<body>Error, invalid value %s<table>", stoday);
      dprintf(idx, "<tr><td><a href=\"../total/\">total</a></td></tr>");
      dprintf(idx, "<tr><td><a href=\"../today/\">today</a></td></tr>");
      dprintf(idx, "</table></body></html>");
      stats_info_access(idx)->code = 404;
      return;
    }
    if (!strcmp(buf, "/") || !buf[0]) {
      dprintf(idx, "<body><table>");
      sprintf(what, "%s", webstats);
      pwhat = what;
      while (strlen(pwhat) > 0) {
        type = newsplit(&pwhat);
        itype = typetoi(type);
        if (itype >= 0) {
          dprintf(idx, "<tr><td><a href=\"%s/\">%s</a></td></tr>", type, type);
        }
      }
      dprintf(idx, "</table></body></html>");
      return;
    }
    gs = findglobstats(channel);
    if (!gs) {
      debug1("Error! Can't find global stats for %s", channel);
      dprintf(idx, "<html><body>Error! Can't find global stats for %s</body></html>", channel);
      stats_info_access(idx)->code = 404;
      return;
    }
    if (!strncasecmp(buf, "graph", 5)) {
      if (graphgif[0] == 0)
        bground[0] = 0;
      else
        sprintf(bground, "background=\"%s\"", graphgif);
      dprintf(idx, "<!-- Created by Stats.mod v%s-->\n", MODULE_VERSION);
      dprintf(idx, "<html><head><title>Top %d statistics %son channel %s</title></head>\n", graphnr, today ? "for today " : "", channel);
      dprintf(idx, "%s\n", bodytag);
      dprintf(idx, "<center><h1>Top %d statistics %son channel %s</h1>\n", graphnr, today ? "for today " : "", channel);
      sprintf(what, "%s", graphstats);
      pwhat = what;
      while (strlen(pwhat) > 0) {
        type = newsplit(&pwhat);
        itype = typetoi(type);
        if (itype < 0) {
          putlog(LOG_MISC, "*", "Stats.mod: Error serving livestats. Unsupported type %s. Skipping.", type);
          continue;
        }
        sortstats(gs, type, today);
        if (gs->local)
          max = gs->local->values[today][itype];
        else {
          debug1("Error! Can't find local stats for %s", channel);
          dprintf(idx, "<html><body>Error! Can't find local stats for %s</body></html>", channel);
	  stats_info_access(idx)->code = 404;
          return;
        }
        total = gettotal(gs, itype, today);
        rest = total;
        dprintf(idx, "<br><br><br><table width=100%% border=1><tr><th width=100%% align=center><font size=+1>Ordered by <b>%s</b></font></td></tr></table>", type);
        dprintf(idx, "<center><font size=-1>[%d %s total]</font></center><br>\n", total, type);
        dprintf(idx, "<table width=100%%>\n");
        onep = (float) total / 100.0;
        nr = 0;
        for (ls = gs->local; ls; ls = ls->next) {
          u = get_user_by_handle(userlist, ls->user);
          if (matchattr(u, nostats, channel))
            continue;
          if (!ls->values[today][itype])
            break;
          nr++;
          if (nr > graphnr)
            break;
          dprintf(idx, "<tr>\n");
          percent = (float) ls->values[today][itype] / onep;
          wpercent = (float) ls->values[today][itype] / ((float) max / 100.0);
          width = (int) wpercent * 0.8;
          dprintf(idx, "  <td align=right width=10%%>%s</td>\n", ls->user);
          dprintf(idx, "  <td align=left width=90%%>\n");
          dprintf(idx, "    <table width=100%%><tr>\n");
          dprintf(idx, "      <td width=%d%% bgcolor=\"%s\" %s>&nbsp;</td>\n", width, graphcolor, bground);
          dprintf(idx, "      <td align=left width=%d%%>%.2f%% <font size=-2>[%d]</font></td>\n", 100 - width, percent, ls->values[today][itype]);
          dprintf(idx, "    </tr></table>\n");
          dprintf(idx, "  </td>\n");
          dprintf(idx, "</tr>\n");
          rest -= ls->values[today][itype];
        }
        dprintf(idx, "<tr>\n");
        percent = (float) rest / ((float) total / 100.0);
        wpercent = (float) rest / ((float) max / 100.0);
        width = (int) wpercent * 0.8;
        dprintf(idx, "  <td align=right width=10%%>[others]</td>\n");
        dprintf(idx, "  <td align=left width=90%%>\n");
        dprintf(idx, "    <table width=100%%><tr>\n");
        dprintf(idx, "      <td width=%d%% bgcolor=\"%s\" %s>&nbsp;</td>\n", width, graphcolor, bground);
        dprintf(idx, "      <td align=left width=%d%%>%.2f%% <font size=-2>[%d]</font></td>\n", 100 - width, percent, rest);
        dprintf(idx, "    </tr></table>\n");
        dprintf(idx, "  </td>\n");
        dprintf(idx, "</tr></table>\n");
      }
      dprintf(idx, "</table>\n");
      dprintf(idx, "<br><center><a href=\"../words/\">back</a><br>Created by <a href=\"http://www.visions-of-fantasy.de/stats.mod/\">Stats.mod v%s</a></center></body></html>", MODULE_VERSION);
      return;
    }
    type = buf;
    itype = typetoi(type);    
    if (itype == T_ERROR) {
      dprintf(idx, "<html><body>Error! No such type %s</body></html>", type);
      stats_info_access(idx)->code = 404;
      return;
    }
    dprintf(idx, "<html><head><title>Top %d talkers %son channel %s</title>\n", webnr, today ? "today " : "", channel);
    dprintf(idx, "<META Name=\"Stats.mod\" Content=\"%s\">\n", MODULE_VERSION);
    dprintf(idx, "<META Name=\"channel\" Content=\"%s\">\n", gs->chan);
    dprintf(idx, "<META Name=\"network\" Content=\"%s\">\n", network);
    dprintf(idx, "<META Name=\"written\" Content=\"%lu\">\n", now);
    dprintf(idx, "</head>\n");
    dprintf(idx, "%s", bodytag);
    dprintf(idx, "<center><h1>Top %d talkers %son channel %s</h1>\n", webnr, today ? "today " : "", channel);
    dprintf(idx, "Ordered by \"%s\" (to change it click corresponding table field).<BR>\n", type);
    dprintf(idx, "User peak: %d<BR>\n", gs->peak[today]);
    dprintf(idx, "<P><table border=2 cellpadding=1><tr align=right>");
    dprintf(idx, "<th>No</th>");
    dprintf(idx, "<th align=center>Nick</th>");
    sprintf(what2, "%s", webstats);
    pwhat2 = what2;
    while (strlen(pwhat2) > 0) {
      type2 = newsplit(&pwhat2);
      itype2 = typetoi(type2);
      if (itype2 == T_MINUTES)
        dprintf(idx, "<th><a href=\"../%s/\">%s</a></th>\n", type2, "time spent");
      else
        dprintf(idx, "<th><a href=\"../%s/\">%s</a></th>\n", type2, type2);
    }
    sortstats(gs, type, today);
    nr = 0;
    for (ls = gs->local; ls; ls = ls->next) {
      u = get_user_by_handle(userlist, ls->user);
      if (matchattr(u, nostats, channel))
        continue;
      if (!ls->values[today][itype])
        break;
      nr++;
      if (nr > webnr)
        break;
      dprintf(idx, "<tr align=right><td>%d</td><td allign=center>%s</td>", nr, ls->user);
      sprintf(what2, "%s", webstats);
      pwhat2 = what2;
      while (strlen(pwhat2) > 0) {
        type2 = newsplit(&pwhat2);
        itype2 = typetoi(type2);
        if (itype2 < 0)
          continue;
        if (itype2 == T_MINUTES)
          dprintf(idx, "<td>%s</td>", duration(ls->values[today][itype2] * 60));
        else
          dprintf(idx, "<td>%d</td>", ls->values[today][itype2]);
      }
      dprintf(idx, "</tr>\n");
    }
    dprintf(idx, "<tr>");
    dprintf(idx, "<td colspan=%d align=center>", countwords(webstats) + 2);
    dprintf(idx, "<b>Total users: %d", countstatmembers(gs));
    dprintf(idx, "</B></td></tr>");
    dprintf(idx, "<tr><td colspan=%d align=center><A HREF=\"../graphs/\">graphs</A></td></tr>", countwords(webstats) + 2);
    if (today)
      dprintf(idx, "<tr><td colspan=%d align=center><A HREF=\"../../total/%s/\">total stats</A></td></tr>", countwords(webstats) + 2, type);
    else
      dprintf(idx, "<tr><td colspan=%d align=center><A HREF=\"../../today/%s/\">stats of today</A></td></tr>", countwords(webstats) + 2, type);
    dprintf(idx, "</table><br>");
    tt = now;
    dprintf(idx, "<a href=\"http://www.visions-of-fantasy.de/stats.mod/\">Stats.mod v%s</a>.<BR>", MODULE_VERSION);
    tt = gs->started;
    strftime(ts, 60, "%d.%m. %Y  %H:%M", localtime(&tt));
    if (!today)
      dprintf(idx, "Started %s<BR>", ts);
    ctime(&ttbuf); /* workaround for eggdrop bug */
    dprintf(idx, "</CENTER>");
    dprintf(idx, "</body></html>");
    return;
  } else if (!strcasecmp(command, "users")) {
    gs = findglobstats(channel);
    if (!gs) {
      debug1("Error! Can't find global stats for %s", channel);
      dprintf(idx, "<html><body>Error! Can't find global stats for %s</body></html>", channel);
      return;
    }
    if (!strcmp(buf, "/") || !buf[0]) {
      sort_stats_alphabetically(gs);
      dprintf(idx, "%s<center><table border=1>\n", bodytag);
      dprintf(idx, "<tr><th>User</th><th>Info</th></tr>\n");
      for (ls = gs->local; ls; ls = ls->next) {
        u = get_user_by_handle(userlist, ls->user);
        if (!u)
          continue;
        what[0] = 0;
        get_handle_chaninfo(ls->user, channel, what);
        pwhat = what;
        if (!what[0])
          pwhat = get_user(&USERENTRY_INFO, u);
        if (!pwhat) {
          sprintf(what, "&nbsp;");
          pwhat = what;
        }
        dprintf(idx, "<tr><td><a href=\"%s/\">%s</a></td><td>%s</td></tr>\n", ls->user, ls->user, pwhat);
      }
      dprintf(idx, "<tr><td colspan=2 align=center><a href=\"http://www.visions-of-fantasy.de/stats.mod/\">");
      dprintf(idx, "Stats.mod v%s</a></td></tr>\n", MODULE_VERSION);
      dprintf(idx, "</table></center></body></html>");
      return;
    }
    user = buf;
    ls = findlocstats(channel, user);
    if (!ls) {
      debug1("Error! Can't find global stats for %s", channel);
      dprintf(idx, "<html><body>I don't have any stats for %s in %s, sorry</body></html>", user, channel);
      stats_info_access(idx)->code = 404;
      return;
    }
    i = countwords(webstats) + 2;
    dprintf(idx, "<html>%s<center>\n", bodytag);
    dprintf(idx, "<table border=1><tr><th colspan=%d align=center>Stats for %s in %s</th></tr>\n", i, ls->user, channel);
    dprintf(idx, "<tr><td colspan=%d>&nbsp;</td> </tr><tr><td></td><td>place</td>", i);
    sprintf(what, "%s", webstats);
    pwhat = what;
    while (strlen(pwhat) > 0) {
      dprintf(idx, "<td>%s</td>", newsplit(&pwhat));
    }
    dprintf(idx, "</tr>\n");
    for (today = 0; today < 2; today++) {
      dprintf(idx, "<tr><td align=center>%s</td>", today ? "today" : "total");
      nr = 0;
      if (ls->values[today][T_WORDS] > 0) {
        sortstats(gs, "words", today);
        for (ls2 = gs->local; ls2; ls2 = ls2->next) {
          nr++;
          if (!strcasecmp(ls2->user, user))
            break;
        }
      }
      if (nr == 0)
        dprintf(idx, "<td>-</td>");
      else
        dprintf(idx, "<td>%d</td>", nr);
      sprintf(what, "%s", webstats);
      pwhat = what;
      while (strlen(pwhat) > 0) {
        itype = typetoi(newsplit(&pwhat));
        if (itype == T_MINUTES)
          dprintf(idx, "<td>%s</td>", duration(ls->values[today][itype] * 60));
        else if (itype == T_ERROR)
          dprintf(idx, "<td>ERR!</td>");
        else
          dprintf(idx, "<td>%d</td>", ls->values[today][itype]);
      }
      dprintf(idx, "</tr>\n");
    }
    dprintf(idx, "<tr><td colspan=%d align=center>\
    	    <a href=\"http://www.visions-of-fantasy.de/stats.mod/\">\
	    Stats.mod v%s</a></td></tr>\n", i, MODULE_VERSION);
    dprintf(idx, "</table></center></body></html>");
    return;
  } else {
    dprintf(idx, "<html><body>Error! unknown command %s</body></html>", command);
    stats_info_access(idx)->code = 404;
    return;
  }
}
