/* 
 * Copyright (C) 2000  Florian Sander
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#define TYPES "words letters started minutes topics lines actions modes bans kicks nicks joins smileys"
#define T_GSTARTED -1
#define T_LSTARTED -2
#define T_PEAK -3
#define T_ERROR -9
#define T_WORDS 0
#define T_LETTERS 1
#define T_MINUTES 2
#define T_TOPICS 3
#define T_LINES 4
#define T_ACTIONS 5
#define T_MODES 6
#define T_BANS 7
#define T_KICKS 8
#define T_NICKS 9
#define T_JOINS 10
#define T_SMILEYS 11
#define TOTAL_TYPES 12

#define S_TODAY 1
#define S_TOTAL 0

typedef struct stats_local {
  struct stats_local *next;
  char *user;
  time_t started;
  int values[2][TOTAL_TYPES];
} locstats;

typedef struct stats_global {
  struct stats_global *next;
  char *chan;
  time_t started;
  int peak[2];
  struct stats_local *local;
} globstats;

struct stats_clientinfo {
  int traffic;
  int code;
  char *browser;
  char *referer;
  char *cmd;
};

#define stats_info_access(i) ((struct stats_clientinfo *) dcc[(i)].u.other)

static void sortstats(struct stats_global *, char *, int);
static int localstats_expmem(struct stats_local *);
static void free_stats();
static void free_localstats(struct stats_local *);
static void reset_tstats();
static void tell_top(char *, char *, char *, int, int, int);
static void tell_place(char *, char *, char *, char *, char *, int);
static void tell_stat(char *, char *, char *, char *, char *, int);
static int typetoi(char *);
static void incrstats(char *, char *, int, int, int);
static int getstats(char *, char *, char *, int);
static locstats *findlocstats(char *, char *);
static globstats *findglobstats(char *);
static int countwords(char *);
static void write_stats();
static void read_stats();
static void writewebstats(char *);
static void write_webstats(char *, int);
static void write_graph_webstats(char *, int);
static int countsmileys(char *);
static int countstatmembers(globstats *);
static int gettotal(globstats *, int, int);
static void stats_autoadd(memberlist *, char *);
static int matchattr(struct userrec *, char *, char *);
static void deloldstatusers();
static void purgestats();
static void sensor_minutely();
static int validchan(char *);
static void start_listen_livestats(int);
static void stop_listen_livestats();
static void display_livestats_accept(int, char *);
static void timeout_livestats(int);
#ifndef OLDBOT
static void outdone_livestats(int);
#endif
static void timeout_listen_livestats(int);
static void livestats_accept(int, char *, int);
static void eof_livestats(int);
static void display_livestats(int, char *);
static void livestats_activity(int, char *, int);
static void send_livestats(int, char *);
char *splitpath(char **);
static void sort_stats_alphabetically(globstats *);
static void sensor_peak(char *);
static int stat_flood();
static void resetlocstats(locstats *);
static int expmem_livestats(void *);
static void kill_livestats(int, void *);
static void out_livestats(int, char *, void *);
static void filt(char *);